module AST.Prolog where

import Data.List (nub)

-- Data structures representing a Prolog program.
-- They are named in a generic way, so I recommend using this module
-- as import qualified.

type Var     = [Char]

type Const   = [Char]


{- type Term    = GeneralTerm Var -}
data Term    = CTVar       Var     -- A bound Variable
	     | CTFreeVar   Int     -- A free Var In the form x1, x2, etc...
	     | CTConst     Const
	     | CTStruct    Const [Term]
	             deriving (Eq, Ord, Show)

data Atom    = CAtom Const [Term]
	             deriving (Eq, Ord, Show)

-- FIXME Literal can be negated

type Literal = Atom

-- Facts and rules are unified, a fact is a rule with 
-- empty body.
-- Clause Head Body

data Clause  = CClause Atom [Literal]
	             deriving (Eq, Show)

-- Name and arity
type Predicate = (Const, Int)

{-

egallego@babel: 23 Sep 2004, 18:22
Monadic implementation of Term data type has been delayed.
For now we use a non monadic data type and manual traversals.


-- The general Term data type.
data GeneralTerm a = CTVar     a
		   | CTFreeVar Int
		   | CTConst   Const
		   | CTStruct  Const [GeneralTerm a]
		     deriving (Eq, Ord, Show)



-- Needs thinking about general traversal of an AST.
-- f is seen as a renaming function.

instance Functor GeneralTerm where
--    fmap :: (a -> b) -> GeneralTerm a -> GeneralTerm b
    fmap f (CTVar v)       = CTVar $ f v
    fmap f (CTStruct n lt) = CTStruct n $ map (fmap f) lt
    fmap f (CTFreeVar n)   = (CTFreeVar n)
    fmap f (CTConst c)     = (CTConst c)



instance Monad GeneralTerm where
    return x              = CTVar x
   
    (CTVar v)       >>= f = f v
    (CTStruct n lt) >>= f = let lt' = map (>>= f) lt in
				(CTStruct n lt')
    (CTConst c)     >>= f = CTConst c
    (CTFreeVar n)   >>= f = CTFreeVar n
-}

-- Utils
arity :: Atom -> Int
arity (CAtom _ l) = length l

getp_rule :: Clause -> Predicate
getp_rule (CClause a _) = getp_atom a

getp_atom :: Atom -> Predicate
getp_atom (CAtom n l) = (n, length l)

vars_clause :: Clause -> [Var]
vars_clause (CClause a ll) = 
	     let va = vars_atom a
	         vl = nub $ concat $ map vars_lit ll in
		      nub $ (va ++ vl)

vars_lit :: Literal -> [Var]
vars_lit l = vars_atom l

vars_atom :: Atom -> [Var]
vars_atom (CAtom _ lt) = nub $ concat $ map vars_term lt

vars_term :: Term -> [Var]
vars_term (CTVar v) = [v]
vars_term (CTConst _ ) = []
vars_term (CTStruct _ lt) = nub $ concat $ map vars_term lt
