-----------------------------------------------------------------
-- |
-- Module      : Insert module
-- Copyright   : (c) Babel Group 2004
-- License     : GPL
--
-- Maintainer  : egallego@babel.ls.fi.upm.es
-- Stability   : medium
-- Portability : portable
-- Last modified: 04 Oct 2004, 18:27
--
-- Manages modules compound of symbols and their code.
-- 
-- This module implementes the data type module,
-- that it is a collection of functions or predicates 
-- and an associated list of definitions.
-----------------------------------------------------------------
module Compiler.Module (
	       Module,
	       emptyModule,
	       addDefinition,
	       mapModule,
	       mapModuleList,
	       moduleToList,
	       lookupCode
) where

import Data.FiniteMap


-- It is an alias for finiteMap, but in the future
-- who knows.

-- | Module type.
--   a is function or predicate type
--   b is code type 
type Module a b = FiniteMap a [b]

-- | Returns an empty module
emptyModule = emptyFM

-- | Adds code to a symbol.
addDefinition :: Ord a =>
		 Module a b  -- ^ Module to modify
	      -> a           -- ^ Function value
	      -> b           -- ^ New code
	      -> Module a b  -- ^ Result module
addDefinition fm name def = addToFM_C (++) fm name [def]

-- |Map for modules.
mapModule :: (a -> b -> c)   -- ^ Function to apply
	  -> Module a b      -- ^ Input Module
	  -> Module a c      -- ^ Output Module
mapModule f fm = 
    let f' = \x -> map (f x) in
	 mapFM f' fm

-- |Apply a function to all codes simultaneously
mapModuleList :: (a -> [b] -> [c]) -- ^ Function to apply
	      -> Module a b        -- ^ Input Module
	      -> Module a c        -- ^ Output Module
mapModuleList f fm = mapFM f fm

-- |Converts a module to a list
moduleToList :: Module a b -> [(a, [b])]
moduleToList fm = fmToList fm

-- |Returns the code associated to a symbol. If the symbol is not 
--  defined, retuns [].
lookupCode :: Ord a => Module a b -> a -> [b]
lookupCode m k = lookupWithDefaultFM m [] k

-- |Join two modules.
combineModules :: Ord a => Module a b -> Module a b -> Module a b
combineModules m1 m2 = plusFM_C (++) m1 m2
