module Engine.Rewritable where

import Control.Monad.State
import Control.Monad.Writer

-- This class encompass a data type that has a state and a type
-- and can rewrite itself. 

-- Note that for the moment, a -> m, this is, each type being
-- reduced are dependant in the data structure behind it.
-- This may change in the future.

class MonadIO m => Rewritable m a where
    isFinal :: a -> m Bool
    rewriteStep :: a -> m a


--
--rewrite :: (MonadWriter [String] m, Rewritable m a, Show a) => a -> m a
rewrite :: Rewritable m a => a -> m a
rewrite d = 
    do f <- isFinal d
       case f of
	    True  -> return d
	    False -> do d' <- rewriteStep d
			rewrite d'

--rewriteN :: (MonadWriter [String] m, Rewritable m a, Show a) => Int -> a -> m a
rewriteN :: Rewritable m a => Int -> a -> m a
rewriteN 0 e = return e
rewriteN n e = 
    do f <- isFinal e
       case f of 
	    True  -> return e
	    False -> do d' <- rewriteStep e
			rewriteN (n-1) d'

-- An example of a rewriter. This rewrites a list 
-- This is a untypical rewriter. In most rewriter engines, the state 
-- will be read-only, as in prolog query is the data to rewrite and 
-- the state is the predicates list. But I don't want to enforce any 
-- of such restricitions until this is more 
{-
instance Rewritable (State Int) [a]  where
    isFinal [] = return True
    isFinal _  = return False
    
    rewriteStep []     = return []
    rewriteStep (x:xs) = do c <- get
			    put (c+1)
			    return xs

testListRewriting s = evalState (rewriteStep s) 0
-}
