module Engine.Unification (
		    unify,
		    apply
) where


--

import AST.Prolog


import Control.Monad.Error

type Substitution = Var -> Term

apply                   :: Substitution -> Term -> Term
apply s (CTVar v)       = s v
apply s t@(CTConst c)   = t
apply s (CTStruct n lt) = CTStruct n (map (apply s) lt) 


subVar :: Var -> Term -> Substitution
subVar v t = \x -> if v == x then t else (CTVar x)

noSub :: Substitution
noSub = \x -> CTVar x

(@@) :: Substitution -> Substitution -> Substitution
s1 @@ s2 = \v -> apply s2 (s1 v) 


-- This is a simple unification algorithm. It returns a substitution 
-- of vars with terms. To apply to a term, we should use apply.
-- Of course, unification can fail, so we returns error 
-- with the MonadError String interface.

unify :: MonadError String m => Term -> Term -> m Substitution
unify t@(CTConst c1) (CTConst c2)
    | c1 == c2  = return noSub
    | otherwise = throwError ("No se puede unificar" ++ (show c1) ++ " con " ++ (show c2))

unify (CTVar v1) e@(CTVar v2)      
    | (v1 == v2) = return noSub 
    | otherwise  = return $ subVar v1 e 

unify (CTVar v) t =
    return $ subVar v t

unify t (CTVar v) =
    return $ subVar v t

unify (CTStruct n1 t1) (CTStruct n2 t2)
  | n1 == n2 && length t1 == length t2 = unifyList t1 t2
  | otherwise                          = throwError ("Lenght error: Cannot unify struct " ++ (show n1) ++ 
						     " with " ++ (show n2))

unify t1 t2 = throwError ("Cannot unify " ++ (show t1) ++ " with " ++ (show t2))


unifyList [] [] = return noSub 
unifyList (x:xs) (y:ys) = 
    do s1 <- unify x y
       s2 <- unifyList (map (apply s1) xs) (map (apply s1) ys) 
       return (s1 @@ s2) 


{-

egallego@babel: 23 Sep 2004, 18:22
Monadic implementation of Term data type has been delayed.
For now we use a non monadic data type and manual traversals.


--bindVar :: (Eq v, Monad m) => v -> m v -> Subst m v
--varBind v t = result $ assoc 

assoc v t = \x -> case x of
			   v       -> t
			   _       -> return x

apply     :: Monad m => (a -> m b) -> m a -> m b
apply s t = t >>= s

(@@) :: Monad m => (a -> m b) -> (c -> m a) -> (c -> m b)
f @@ g = \x -> do
	       y <- g x
	       f y

--gunify t1 t2 = 
  --  apply (unify t1 t2) t1


unify (CTConst c1) (CTConst c2)
 | c1 == c2  = return return
 | otherwise = fail ("No se puede unificar" ++ (show c1) ++ " con " ++ (show c2))

unify (CTVar v1) e@(CTVar v2)      
    | (v1 == v2) = return return 
    | otherwise  = return $ assoc v1 e 

unify (CTVar v) t =
    return $ assoc v t

unify t (CTVar v) =
    return $ assoc v t

unify (CTStruct n1 t1) (CTStruct n2 t2)
  | n1        == n2     && 
    length t1 == length t2 = unifyList t1 t2
  | otherwise              = fail ("Cannot unify struct " ++ (show n1) ++ 
				   " with " ++ (show n2))

unify t1 t2 = fail ("Cannot unify " ++ (show t1) ++ " with " ++ (show t2))

--
--unifyList :: [GeneralTerm a] -> [GeneralTerm a] -> Subst GeneralTerm a
unifyList [] [] = return 
unifyList (x:xs) (y:ys) = 
    do s1 <- unify x y
       s2 <- unifyList (map (apply s1) xs) (map (apply s1) ys) 
       return (s1 @@ s2) 


--data Term    = CTVar       Var     -- A bound Variable
--	     | CTFreeVar   Int     -- A free Var In the form x1, x2, etc...
--	     | CTConst     Const
--	     | CTStruct    Const [Term]
--	             deriving (Eq, Ord, Show)
-}
