module Main where

import System
import Data.Maybe

import Shell.Options
import Shell.Compiler

import Pretty.Pretty

import Pretty.Ralog
import Pretty.Prolog
import Pretty.Relational

import Shell.Interactive

import AST.Prolog

-- Main Loop

main = do args <- getArgs
	  ramc args

-- Relational Abstract Machine Compiler
ramc    :: [String] -> IO ()
ramc args = do
    (opts, file) <- getOptions args
    let isInteractive = isJust $ getFlag "Interactive" opts 
	isHelp        = isJust $ getFlag "Help"        opts in
	if isInteractive 
	   then interactiveShell file 
	   else 
	if isHelp
	   then putStr help
	   else 
	batchCompiler opts file

batchCompiler :: [Flag] -> [String] -> IO ()
batchCompiler _ [] = putStr help
batchCompiler opts (f:[]) = 
    let isRalog           = isJust                      $ getFlag "Ralog"  opts 
        (PString outFile) = fromMaybe (PString "r.out") $ getFlag "Output" opts in
		  do pText <- readFile f
		     if not isRalog then
			case (compiler1 pText) of
			     Left err  -> putStr (err++"\n")
			     Right exp -> writePretty outFile exp
		        else
		        case (compiler2 pText) of
			     Left err  -> putStr (err++"\n")
			     Right exp -> writePretty outFile exp

batchCompiler _ xs = putStr ("Too much files to compile\n" ++ help)   

compiler1 t = do pro <- compileToProlog t
		 rel <- compileToRelational pro
		 return rel 

compiler2 t = do pro <- compileToProlog t
		 rel <- compileToRelational pro
		 ral <- compileToRalog rel
		 return ral

writePretty :: Pretty a => String -> a -> IO ()
writePretty file p = writeFile file (pretty p)

