module Pretty.Prolog where

import AST.Prolog

import Text.PrettyPrint

import Compiler.Module (Module, moduleToList)

import Pretty.Pretty

--


instance Pretty (Module Predicate Clause) where
    pretty = concat . prettyProlog

prettyProlog :: (Module Predicate Clause) -> [String]
prettyProlog fm = 
    let lr = moduleToList fm in 
	     map pretty' lr

pretty' :: (Predicate, [Clause]) -> String
pretty' (p, lc) = render $ ppPred p $$ nest 4 (vcat (map ppClause lc)) <> text "\n"

ppPred :: Predicate -> Doc
ppPred (p, n) = text p <> text "/" <> int n

ppClause :: Clause -> Doc
ppClause (CClause a l) = ppAtom a <+> text ":-" <+>
			 (hsep (punctuate comma (map ppAtom l))) <+> text "."

ppAtom :: Atom -> Doc
-- Ugly hack FIXME
ppAtom (CAtom "=" [l1, l2]) = ppTerm l1 <> text "=" <> ppTerm l2
ppAtom (CAtom n lt) = text n <> parens (hsep (punctuate comma (map ppTerm lt)))

ppTerm :: Term -> Doc
ppTerm (CTVar v)         = ppVar v
ppTerm (CTFreeVar n)     = text ("_x"++show(n))
ppTerm (CTConst c)       = ppConst c
ppTerm s@(CTStruct _ _)  = ppStruct s

ppVar :: Var -> Doc
ppVar n = text n 
 
ppConst :: Const -> Doc
ppConst n = text n

ppStruct :: Term -> Doc
ppStruct (CTStruct n lt) = text n <> 
			   parens (hsep (punctuate comma (map ppTerm lt)))
