module Pretty.Relational where

import Text.PrettyPrint

import AST.Prolog
import AST.Vector
import AST.Relational

import Compiler.Module (Module, moduleToList)

import Pretty.Pretty

import qualified Pretty.Prolog as PP

--

instance Pretty (Module Predicate Relational) where
    pretty = concat . prettyRelational

instance Pretty Relational where
    pretty r = render $ ppRel r

prettyRelational :: (Module Predicate Relational) -> [String]
prettyRelational fm = 
    let lr = moduleToList fm in
	     map ppRelPred lr

ppRelPred :: (Predicate, [Relational]) -> String
ppRelPred (p, r) = render $ ppPred' p $$ nest 2 (vcat (map ppRel r)) <> text "\n"

ppPred' :: Predicate -> Doc
ppPred' (p, n) = text p <> text "/" <> int n
	
ppRel :: Relational -> Doc
ppRel (CUnion l)        = brackets $ vcat (punctuate (text " U") (map ppRel l))
ppRel (CIntersection l) = brackets $ vcat (punctuate (text " /\\") (map ppRel l))
ppRel (CConcat l)       = brackets $ hsep (punctuate (text " .") (map ppRel l))

ppRel (FK v)    = text "K" <> ppVector v
ppRel (FW s)    = ppPerm s
ppRel (FPred s) = ppPred' s
ppRel (FConv f) = text "cnv" <> parens (ppRel f)
ppRel (FI i)    = text $ "I" ++ show i
ppRel (F0)      = text $ "0"

ppPerm :: Perm -> Doc
ppPerm (CPerm v1 v2) = text "W" -- <> (parens $ ppVector v1 <> text "->" <> ppVector v2)

ppVector :: Vector -> Doc
ppVector l = parens (hsep (punctuate comma (map PP.ppTerm l)))
