-----------------------------------------------------------------
-- |
-- Module      : Shell.Interactive
-- Copyright   : (c) Babel Group 2004
-- License     : GPL
--
-- Maintainer  : egallego@babel.ls.fi.upm.es
-- Stability   : experimental
-- Portability : portable
-- Last modified: 04 Oct 2004, 18:42
--
-- This module implements an interactive Shell for ram.
-- 
-----------------------------------------------------------------
module Shell.Interactive where

import Data.Either

import Control.Monad.State

import System.IO
import System.Console.Readline as Readline

--
import Shell.Compiler (compileToProlog,
		       compileToRelational)

-- Maybe this should go away.
import qualified Shell.Options as Opts

import Pretty.Pretty
import Pretty.Prolog
import Pretty.Relational

import Shell.Interactive.Lexer
import Shell.Interactive.Parser
import Shell.Interactive.Command

import Compiler.Module

import AST.Prolog
import AST.Relational

import Engine.Rewritable
import Engine.Relational

-- Monad for interactive shell

data ShellState = ShellState 
     { 
      memory :: Module Predicate Relational
     }

compileraux t = do pro <- compileToProlog t
		   rel <- compileToRelational pro
		   return rel 

header = "Relational Abstract Machine Prolog Compiler" ++
         "Copyright (C) 2004 Babel Group\n\n" ++
         "This program is free software; you can redistribute it and/or \n"  ++
         "modify it under the terms of the GNU General Public License   \n"  ++
         "as published by the Free Software Foundation; either version 2\n"  ++
         "of the License, or (at your option) any later version.      \n\n"  ++
	 "This program is distributed in the hope that it will be useful,\n" ++
         "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"  ++
         "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n"  ++
         "GNU General Public License for more details.                   \n\n"

-- Hack, hack.
type IModule = Module Predicate Relational




-- Gets a file to compile and launches queries against it.
interactiveShell :: [String] -> IO ()
interactiveShell [] = putStr "Please specify a file to load\n"
interactiveShell (f:_) = do
    file <- readFile f

    case compileraux file of
        Left err -> putStr (err ++ "\n")
	Right m -> do
	           -- Setting buffers.
		   hFlush stdout
		   hSetBuffering stdout NoBuffering
				 
                   hSetBuffering stdin  NoBuffering
		   
		   --  options
		   Readline.initialize

		   putStr header
	  
                   shellLoop m
		   
	           Readline.resetTerminal Nothing
 
	           return ()

-- We should use a state monad to pass the module
shellLoop :: IModule -> IO ()
shellLoop m = do 
    cmd <- Readline.readline ":- "
    
    case cmd of
	 Just cmd -> do Readline.addHistory cmd
			-- Aqu ir nuestra accin.
			doCommand m parsed
			shellLoop m
			    where parsed = interactive_parser (parseToken cmd)
	 Nothing  -> return ()
	     

-- Should be doCommand :: Command -> IShell ()
-- so using a custom monad
--doCommand                  :: Command -> IO ()
doCommand :: IModule -> Command -> IO ()
doCommand _ Empty            = return ()
doCommand _ (ShellCommand c) = return ()
-- Its a query, so pass :- to compiler.
--doCommand m (Query s)        = executeQuery m (":- " ++ s)
doCommand m (Query s)        = executeQuery m s


executeQuery :: IModule -> String -> IO ()
executeQuery m program = 
    case r of 
	   Left  e -> putStr (e ++ "\n")
	   Right q -> executeQueryEngine m $ mkQuery q 
  where r = do q <- compileToProlog program
	       p <- compileToRelational q
	       return p

executeQueryEngine :: IModule -> Relational -> IO ()
executeQueryEngine m query =
    do putStr (pretty query)
       final <- isFinalHack m query
       case final of
            True -> do putStr "\n  Final expression. Rewrite more? (y/n)? "
		       resp <- getChar
		       putStr "\n"
		       case resp of
		            'y' -> do expr <- rewriteStepHack m query
				      executeQueryEngine m expr
			    _   -> return ()
	    False -> do putStr "\n------\n"
			c <- getChar
			case c of
			     's' -> return ()
			     _   -> do expr <- rewriteStepHack m query
				       executeQueryEngine m expr


isFinalHack :: IModule -> Relational -> IO Bool
isFinalHack m expr =
    let rewriteST = isFinal expr in
	evalStateT rewriteST m

rewriteStepHack :: IModule -> Relational -> IO Relational
rewriteStepHack m expr = 
    let rewriteST = rewriteStep expr in
	evalStateT rewriteST m
    

-- Get query and launch it. 
-- By now, we get the first predicate and it is the query,
-- but obviously this needs fixing.
mkQuery :: Module Predicate Relational -> Relational
mkQuery r = 
    let ( (pred, p):_) = moduleToList r 
	(x:xs) = take 1 p                
	(CUnion (y:ys)) = x               in
	CIntersection $ [y, FPred pred]


{- Future :print expr.
doCommand (Query s) =
    do putStr $ (queryPrint s) ++ "\n"
       return ()       
-}

-- | queryPrint compiles a query and prints
-- its relational translation.
queryPrint :: String -> String 
queryPrint s = 
    case r of 
	   Left e -> e
	   Right c -> pretty c
    where r = do q <- compileToProlog s
		 p <- compileToRelational q
		 return p
