-----------------------------------------------------------------
-- |
-- Module      : Shell.Options
-- Copyright   : (c) Babel Group 2004
-- License     : GPL
--
-- Maintainer  : egallego@babel.ls.fi.upm.es
-- Stability   : Minor changes allowed.
-- Portability : portable
-- Last modified: 22 Oct 2004, 17:41
--
-- Process command line arguments.
-- 
-----------------------------------------------------------------
module Shell.Options (
		      Params(PInt, PString, PNothing), 
		      Flag(Flag),
		      getFlag,    
		      getOptions, 
		      help        
) where

import Data.List
import Data.Maybe
import System.Console.GetOpt

-- | A command line flag can have some data associated to it.
data Params = PInt Int         
	    | PString String
	    | PNothing
	      deriving (Eq, Show)

-- | Flag data type.
data Flag = Flag String Params 
	    deriving (Eq, Show)

-- | 
-- Based on GetOpt documentation.
-- Returns the list of command line flags and arguments
-- We use IO monad for error purposes.

getOptions :: [String]              -- ^ Command line args.
	   -> IO ([Flag], [String])
getOptions argv =  
    case (getOpt RequireOrder options argv) of
        (f, n, [])   -> return (f, n)
	(_, _, errs) -> ioError (userError (concat errs ++ usageInfo header options))

-- |
-- Help string of ramc.
help :: String
help = usageInfo header options

header :: String
header = "Relational Abstract Machine Prolog Compiler\n" ++
	 "(c) Babel Group 2004:\n\n" ++
	 "    Gallego Arias, Emilio Jess\n" ++
	 "    Lipton, James\n" ++
	 "    Mario Carballo, Julio\n\n" ++
	 "Usage: ramc [OPTIONS] file"

options :: [OptDescr Flag]
options = 
    --Option ['v']    ["verbose"]     (OptArg (mkFlag "Verbose" . (readNumberDefault 0)) "n")  "verbose level (obsolete)"
    --, Option ['p']    ["pretty"]      (OptArg (mkFlag "Pretty"  . (readNumberDefault 1)) "n")  "pretty print level (obsolete)"
    [ Option ['i']    ["interactive"] (NoArg  (mkFlag "Interactive" PNothing )              )  "interactive shell" 
    , Option ['h']    ["help"]        (NoArg  (mkFlag "Help"        PNothing )              )  "show help"
    , Option ['r']    ["ralog"]       (NoArg  (mkFlag "Ralog"       PNothing )              )  "output ralog code"
    , Option ['o']    ["output"]      (ReqArg (mkFlag "Output"    . PString  )        "FILE")  "output filename, default r.out"
    ]


mkFlag :: String -> Params -> Flag 
mkFlag n p = Flag n p

readNumberDefault :: Int -> Maybe String -> Params
readNumberDefault dn s = PInt $ read (fromMaybe (show dn) s)

-- |
-- Gets flag parameters.
getFlag :: String -- ^ Name of flag to search
	-> [Flag] -- ^ List of flags.
	-> Maybe Params -- ^ Flag parameters.
getFlag s l = case search of
		   Just (Flag _ p) -> Just p
		   Nothing         -> Nothing
    where search = find (\(Flag n p) -> s==n) l
