module Translator.Relational.Ralog where

import AST.Prolog
import AST.Vector
import AST.Relational
import qualified AST.Ralog as Ralog

import Compiler.Module (Module, mapModule)
import Translator


instance Translator (Module Predicate Relational) Predicate Ralog.Ralog where
    translate a = Right $ trans a


trans :: Module Predicate Relational
      -> Module Predicate Ralog.Ralog
trans rp = 
    mapModule translatePredicate rp

translatePredicate :: Predicate -> Relational -> Ralog.Ralog
translatePredicate p r = translateR r

translateR :: Relational -> Ralog.Ralog
translateR (CUnion        l) = Ralog.CUnion $ map translateR l
translateR (CIntersection l) = Ralog.CIntersection $ map translateR l
translateR (CConcat       l) = Ralog.CConcat $ map translateR l

translateR (FK    v)      = translateKo v
translateR (FI    i)      = translateI i
translateR (FConv f) = 
                          let ft = translateR f in
	                           Ralog.COp $ Ralog.Conv ft
translateR (FW    p)      = translateW p
translateR (FPred (n, a)) = Ralog.COp $ Ralog.RelPred n

--

-- This should go in Utils
-- map with the number in the list.
mapWithNumber :: (Int -> a -> b) -> [a] -> [b]
mapWithNumber = mapWithNumber' 1

mapWithNumber' :: Int -> (Int -> a -> b) -> [a] -> [b]
mapWithNumber' n f [] = []
mapWithNumber' n f (x:xs) = (f n x) : (mapWithNumber' (n+1) f xs)
	      
--
translateKo   :: Vector -> Ralog.Ralog
translateKo v = 
    let kt = translateK v in
	   Ralog.CIntersection [Ralog.COp Ralog.ID, Ralog.CConcat [kt, Ralog.COp Ralog.One]]

translateK :: Vector -> Ralog.Ralog
translateK v = Ralog.CIntersection $ mapWithNumber translateKUnit v

translateKUnit :: Int -> Term -> Ralog.Ralog
translateKUnit n t = Ralog.CConcat [translateP n, translateKTerm t]

translateKTerm :: Term -> Ralog.Ralog
translateKTerm (CTConst c)   = Ralog.CConcat [ Ralog.COp $ Ralog.Relation [(c,c)], Ralog.COp Ralog.One]
translateKTerm (CTVar   v)   = Ralog.CIntersection [] -- never possible.
translateKTerm (CTFreeVar i) = Ralog.COp $ Ralog.Conv (translateP i)
translateKTerm (CTStruct  p t) = Ralog.CIntersection $ mapWithNumber (translateKStruct p (length t)) t

-- 
translateKStruct :: Const -> Int ->  Int -> Term -> Ralog.Ralog
translateKStruct p l n t = Ralog.CConcat [ project p l n, translateKTerm t]

-- 
-- Generates a projection.
project :: Const -> Int -> Int -> Ralog.Ralog
project p l n = Ralog.COp $ Ralog.Projection p l n

--
-- Generates a P.
translateP :: Int -> Ralog.Ralog
translateP n = Ralog.CConcat $ (replicate (n-1) $ Ralog.COp Ralog.TL) ++ [Ralog.COp Ralog.HD]

-- Translates a I.
translateI :: Int -> Ralog.Ralog
translateI n = Ralog.CIntersection $ map intToP [1..n]

intToP :: Int -> Ralog.Ralog
intToP n = Ralog.CConcat [translateP n, Ralog.COp $ Ralog.Conv (translateP n)]

-- Of moment we assume that all the perms are in the form [x1..xn] [xi..xj]
-- That it's, the first vector is ordered.
translateW :: Perm -> Ralog.Ralog
translateW (CPerm _ v) = 
    Ralog.CIntersection $ mapWithNumber genPerm v
	 where
	 genPerm nj (CTFreeVar ni) = Ralog.CConcat [translateP ni, Ralog.COp $ Ralog.Conv (translateP nj)]


