{

-- This module implements the parsing of a Prolog grammar.

module Translator.String.Prolog.Syntax where

import AST.Prolog

import Translator.String.Prolog.Lexer

import Compiler.Module (Module, addDefinition)

import Data.Either
import Control.Monad.Error
import Control.Monad.State

}

%name prolog_parser

%tokentype { PrologToken }

%monad { PrologParseMonad } --{ (>>=) } { returnM }

-- Tokens of Prolog.
%token
      '.'             { TSym "."  }
      ','             { TSym ","  }
      ':-'            { TSym ":-" }
      '('             { TSym "("  }
      ')'             { TSym ")"  }
      VAR             { TVar $$   } 
      CONST           { TConst $$ }

%%
-- Grammar of EONF descriptions.

program :: { [Predicate] }
    : clause                     { [$1] }
    | program clause             { $1 ++ [$2] }

clause  :: { Predicate }
    : rule                       { % addClause (getp_rule $1) $1 }
    | atom '.'                   { % addClause (getp_atom $1) (CClause $1 []) }

rule    :: { Clause }
    : atom ':-' rule_body '.'    { CClause $1 $3 }
    | ':-' rule_body '.'         { CClause (mkQuery $2) $2 }

rule_body :: { [Literal] }
    : literal                    { [$1] }
    | rule_body ',' literal      { $1 ++ [$3] }


literal :: { Literal }
    : atom                       { $1 }

atom    :: { Atom    } 
    : CONST '(' term_list ')'    { CAtom $1 $3 }
    | CONST                      { CAtom $1 [] }

term_list :: { [Term] }
    : term                       { [$1] }
    | term_list ',' term         { $1 ++ [$3] }

term    :: { Term }
    : VAR                        { CTVar    $1 }
    | CONST                      { CTConst  $1 }
    | struct                     { $1 }

struct  :: { Term }
    : CONST '(' term_list ')'    { CTStruct $1 $3 }



{

-------------------------
-- | Parser State monads.
--

type ParserState = Module Predicate Clause

type PrologParseMonad a = ErrorT [Char] (State ParserState) a

-- FIXME: We should use monad transformers.
--returnM :: a -> PrologParseMonad a
--returnM a = return $ return a

--thenM :: PrologParseMonad a -> (a -> PrologParseMonad b) -> PrologParseMonad b
--thenM e f = case e of 
--		   Right a -> f 
--		   Left  b -> Left b

-- Predicate query
mkQuery :: [Literal] -> Atom
mkQuery l = 
    let d  = CClause (CAtom "query" []) l 
	nv = vars_clause d     
	lt = map (\x -> CTVar x) nv in
	     CAtom "query" lt

-- Adds a clause to a predicate.

addClause :: Predicate -> Clause -> PrologParseMonad Predicate
addClause p c = lift $ do mod <- get
			  put (addDefinition mod p c)
			  return p

happyError :: [PrologToken] -> PrologParseMonad a
happyError _ = fail "Compile error"

}
