module Util.StrictMap where

-- This implements a data structure with Strict 1 <-> 1 mapping.
-- It's used in eliminating x2 = a, x3 = a in prolog.

import Data.FiniteMap

data StrictMap a b = SM (FiniteMap a b) (FiniteMap b a)

reverseSM :: StrictMap a b -> StrictMap b a
reverseSM (SM fm1 fm2) = SM fm2 fm1

emptySM :: StrictMap a b 
emptySM = SM emptyFM emptyFM


addToSM   :: (Ord a, Ord b) => StrictMap a b -> a -> b -> StrictMap a b
addToSM sm@(SM fm1 fm2) o1 o2 = 
    let fm1'    = addToFM fm1 o1 o2
	fm2'    = addToFM fm2 o2 o1 in
	   SM fm1' fm2'
	

delFromFirstSM  :: (Ord a, Ord b) => StrictMap a b -> a -> StrictMap a b
delFromFirstSM sm@(SM fm1 fm2) o =
    case lookupFM fm1 o of
         Nothing -> sm
	 Just o' -> let fm1'  = delFromFM fm1 o
			fm2'  = delFromFM fm2 o' in
			   SM fm1' fm2'


delFromSecondSM :: (Ord a, Ord b) => StrictMap a b -> b -> StrictMap a b
delFromSecondSM sm@(SM fm1 fm2) o = 
    let sm' = reverseSM sm in
	  reverseSM $ delFromFirstSM sm' o

lookupFirstSM   :: (Ord a, Ord b) => StrictMap a b -> a -> Maybe b
lookupFirstSM sm@(SM fm1 fm2) o =
    lookupFM fm1 o 

lookupSecondSM  :: (Ord a, Ord b) => StrictMap a b -> b -> Maybe a
lookupSecondSM sm@(SM fm1 fm2) o =
    lookupFM fm2 o



-- Un StrictMap solo puede mapearse con una funcin si es del mismo tipo.
-- Por eso hacemos un map de dos funciones.
mapSM :: (Ord a, Ord b) => StrictMap a b -> (a -> b -> b) -> (b -> a -> a) -> StrictMap a b
mapSM sm@(SM fm1 fm2) f1 f2 = 
    let fm1' = mapFM f1 fm1
	fm2' = mapFM f2 fm2 in
	   SM fm1' fm2'

smToList        :: StrictMap a b -> [(a,b)]
smToList (SM fm1 _) = fmToList fm1


listToSM :: (Ord a, Ord b) => [(a, b)] -> StrictMap a b
listToSM l =
    let fm1 = listToFM l
	fm2 = listToFM $ map (\(a,b) -> (b,a)) l in
	      SM fm1 fm2
